﻿using Enterprise_Store.Data;
using System;
using System.Collections.Generic;
using Windows.UI.Xaml.Controls;
using Windows.UI.Popups;
using Windows.UI.ApplicationSettings;
using Enterprise_Store.Common;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Navigation;

// The Group Detail Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234229

namespace Enterprise_Store
{
    /// <summary>
    /// A page that displays an overview of a single group, including a preview of the items
    /// within the group.
    /// </summary>
    public sealed partial class GroupDetailPage : Page
    {

        private NavigationHelper navigationHelper;
        private ObservableDictionary defaultViewModel = new ObservableDictionary();

        public static GroupDetailPage Current;

        /// <summary>
        /// NavigationHelper is used on each page to aid in navigation and 
        /// process lifetime management
        /// </summary>
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        /// <summary>
        /// This can be changed to a strongly typed view model.
        /// </summary>
        public ObservableDictionary DefaultViewModel
        {
            get { return this.defaultViewModel; }
        }


        public GroupDetailPage()
        {
            this.InitializeComponent();

            Current = this;

            SettingsPane.GetForCurrentView().CommandsRequested += SettingCharmManager_CommandsRequested;
            this.navigationHelper = new NavigationHelper(this);
            this.navigationHelper.LoadState += navigationHelper_LoadState;
            Window.Current.SizeChanged += WindowSizeChanged;
        }


        private void WindowSizeChanged(object sender, Windows.UI.Core.WindowSizeChangedEventArgs e)
        {
            manageDisplay(e.Size.Width);
        }

        private void manageDisplay(double Width)
        {
            if (Width <= 500)
            {
                itemListView.Visibility = Visibility.Visible;
                itemGridView.Visibility = Visibility.Collapsed;
                pageTitleSnapped.Visibility = Visibility.Visible;
                pageTitle.Visibility = Visibility.Collapsed;
                backButton.Visibility = Visibility.Collapsed;
                backButtonSnapped.Visibility = Visibility.Visible;

                if (((SampleDataGroup)DefaultViewModel["Group"]).UniqueId == "Search")
                {
                    homeButton.Visibility = Visibility.Collapsed;
                    homeButtonSnapped.Visibility = Visibility.Visible;
                }

            }
            else if (Width <= 900)
            {
                GroupImage.Visibility = Visibility.Collapsed;

                itemListView.Visibility = Visibility.Collapsed;
                itemGridView.Visibility = Visibility.Visible;
                pageTitleSnapped.Visibility = Visibility.Collapsed;
                pageTitle.Visibility = Visibility.Visible;
                backButton.Visibility = Visibility.Visible;
                backButtonSnapped.Visibility = Visibility.Collapsed;

                if (((SampleDataGroup)DefaultViewModel["Group"]).UniqueId == "Search")
                {
                    homeButton.Visibility = Visibility.Visible;
                    homeButtonSnapped.Visibility = Visibility.Collapsed;
                }
            }
            else
            {
                GroupImage.Visibility = Visibility.Visible;

                itemListView.Visibility = Visibility.Collapsed;
                itemGridView.Visibility = Visibility.Visible;
                pageTitleSnapped.Visibility = Visibility.Collapsed;
                pageTitle.Visibility = Visibility.Visible;
                backButton.Visibility = Visibility.Visible;
                backButtonSnapped.Visibility = Visibility.Collapsed;

                if (((SampleDataGroup)DefaultViewModel["Group"]).UniqueId == "Search")
                {
                    homeButton.Visibility = Visibility.Visible;
                    homeButtonSnapped.Visibility = Visibility.Collapsed;
                }
            }

            if (((SampleDataGroup)DefaultViewModel["Group"]).ImagePath == "")
            {
                GroupImage.Visibility = Visibility.Collapsed;
            }
        }

        private void SettingCharmManager_CommandsRequested(SettingsPane sender, SettingsPaneCommandsRequestedEventArgs args)
        {
            if (App.displayPrivacyPolicy)
            {
                args.Request.ApplicationCommands.Clear();
                SettingsCommand cmd = new SettingsCommand("privacypolicy", "Privacy policy", (uiCommand) =>
                {
                    OpenPrivacyPolicy();
                });

                args.Request.ApplicationCommands.Add(cmd); 
                
                cmd = new SettingsCommand("getthisapp", "Get this app for your customer", (uiCommand) =>
                {
                    GetThisApp();
                });

                args.Request.ApplicationCommands.Add(cmd);
            }
        }

        private async void OpenPrivacyPolicy()
        {
            Uri uri = new Uri("http://companystore.codeplex.com/wikipage?title=Privacy&version=1");
            await Windows.System.Launcher.LaunchUriAsync(uri);
        }

        private async void GetThisApp()
        {
            Uri uri = new Uri("http://companystore.codeplex.com/");
            await Windows.System.Launcher.LaunchUriAsync(uri);
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        private void navigationHelper_LoadState(object sender, LoadStateEventArgs e)
        {
            if (e.NavigationParameter != null) 
            {
                // TODO: Create an appropriate data model for your problem domain to replace the sample data
                var group = SampleDataSource.GetGroup((String)e.NavigationParameter);
                this.DefaultViewModel["Group"] = group;
                this.DefaultViewModel["Items"] = group.Items;

                manageDisplay(Window.Current.Bounds.Width);
            }
        }

        /// <summary>
        /// Invoked when an item is clicked.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is snapped)
        /// displaying the item clicked.</param>
        /// <param name="e">Event data that describes the item clicked.</param>
        async void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            // Navigate to the appropriate destination page, configuring the new page
            // by passing required information as a navigation parameter
            String uri = ((SampleDataItem)e.ClickedItem).Description;

            var option = new Windows.System.LauncherOptions();
            if (((SampleDataItem)e.ClickedItem).Content != "")
            {
                option.PreferredApplicationPackageFamilyName = ((SampleDataItem)e.ClickedItem).Content;
                option.PreferredApplicationDisplayName = ((SampleDataItem)e.ClickedItem).Title;
            }
            if (uri.ToUpper().Contains("daipp://windowsStoreWin8app".ToUpper()) && !uri.ToUpper().Contains("storeid=".ToUpper()))
            {
                uri += "?storeid=" + ((SampleDataItem)e.ClickedItem).Content;
            }
            try
            {
                //If the URI is well formated go on, if not display the error message.                
                if (Uri.IsWellFormedUriString(uri, UriKind.Absolute) || uri.ToUpper().Contains("daipp://".ToUpper()))
                {
                    await Windows.System.Launcher.LaunchUriAsync(new Uri(uri), option);
                }
                else
                {
                    ((SampleDataItem)e.ClickedItem).Subtitle = String.Format(App.errorURIMessage, uri);
                    if (((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed)
                    {
                        ((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed = true;
                    }
                }
            }
            catch (FormatException)
            {
                ((SampleDataItem)e.ClickedItem).Subtitle = String.Format(App.errorURIMessage, ((SampleDataItem)e.ClickedItem).Description);
                if (((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed)
                {
                    ((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed = true;
                }
            }
        }


        public void DisplaySearchResult(String search)
        {
            SampleDataGroup res = SampleDataSource.GetItems(search);
            if (res != null)
            {
                this.DefaultViewModel["Group"] = res;
                this.DefaultViewModel["Items"] = res.Items;
            }
            else 
            { 
            }

            manageDisplay(this.Width);
        }

        private void goHome(object sender, RoutedEventArgs e)
        {
            var rootFrame = new Frame();
            rootFrame.Navigate(typeof(GroupedItemsPage));
            Window.Current.Content = rootFrame;

            Window.Current.Activate();
        }

        #region NavigationHelper registration

        /// The methods provided in this section are simply used to allow
        /// NavigationHelper to respond to the page's navigation methods.
        /// 
        /// Page specific logic should be placed in event handlers for the  
        /// <see cref="GridCS.Common.NavigationHelper.LoadState"/>
        /// and <see cref="GridCS.Common.NavigationHelper.SaveState"/>.
        /// The navigation parameter is available in the LoadState method 
        /// in addition to page state preserved during an earlier session.

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            navigationHelper.OnNavigatedFrom(e);
        }

        #endregion
    }
}
