﻿using Enterprise_Store.Data;

using System;
using System.Collections.Generic;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Popups;
using Windows.UI.ApplicationSettings;
using System.Collections.ObjectModel;
using Windows.Management.Deployment;
using Windows.Foundation;
using System.Threading;
using Windows.Data.Xml.Dom;
using Windows.UI.Notifications;


// The Grouped Items Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234231

namespace Enterprise_Store
{
    /// <summary>
    /// A page that displays a grouped collection of items.
    /// </summary>
    public sealed partial class GroupedItemsPage : Enterprise_Store.Common.LayoutAwarePage
    {
        public static String feed1Name = "";
        public static String feed1URI = "";
        public static String feed2Name = "";
        public static String feed2URI = "";

        
        public GroupedItemsPage()
        {
            this.InitializeComponent();
            SettingsPane.GetForCurrentView().CommandsRequested += SettingCharmManager_CommandsRequested;
        }

        private void SettingCharmManager_CommandsRequested(SettingsPane sender, SettingsPaneCommandsRequestedEventArgs args)
        {
            if (App.displayPrivacyPolicy)
            {
                args.Request.ApplicationCommands.Clear();
                SettingsCommand cmd = new SettingsCommand("privacypolicy", "Privacy policy", (uiCommand) =>
                {
                    OpenPrivacyPolicy();
                });

                args.Request.ApplicationCommands.Add(cmd);

                cmd = new SettingsCommand("getthisapp", "Get this app for your customer", (uiCommand) =>
                {
                    GetThisApp();
                });

                args.Request.ApplicationCommands.Add(cmd);
            }
        }

        private async void OpenPrivacyPolicy()
        {
            Uri uri = new Uri("http://companystore.codeplex.com/wikipage?title=Privacy&version=1");
            await Windows.System.Launcher.LaunchUriAsync(uri);
        }

        private async void GetThisApp()
        {
            Uri uri = new Uri("http://companystore.codeplex.com/");
            await Windows.System.Launcher.LaunchUriAsync(uri);
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
            double tp = ((Window.Current.Bounds.Height - 206) / 120);
            
            String error = SampleDataSource.getError();
            if (error != null)
            {
                showError(error);
            }

            // TODO: Create an appropriate data model for your problem domain to replace the sample data
            var sampleDataGroups = SampleDataSource.GetGroups((String)navigationParameter);
            this.DefaultViewModel["Groups"] = sampleDataGroups;

            //Display Feeds if there is valid feeds defined in the storestructure.xml
            if ((int)Application.Current.Resources["nbFeeds"]==0)
            {
                this.ContentGrid.ColumnDefinitions.Clear();
                var col1 = new ColumnDefinition();

                col1.Width = new GridLength(0);
                this.ContentGrid.ColumnDefinitions.Add(col1);

                var col2 = new ColumnDefinition();

                this.ContentGrid.ColumnDefinitions.Add(col2);


                this.itemGridView.Padding = new Thickness(116, 0, 0, 0);
            }
            else
            {
                var feedsDataGroups = FeedDataSource.GetGroups((String)navigationParameter, (int)Math.Round(tp / (int)Application.Current.Resources["nbFeeds"]));
                this.DefaultViewModel["RSSFeed"] = feedsDataGroups;
            }

            //set the new margin for the main grids
            string[] margin = App.itemGridViewMargin.Split(',');

            try
            {
                this.itemGridView.Margin = new Thickness(Double.Parse(margin[0]), Double.Parse(margin[1]), Double.Parse(margin[2]), Double.Parse(margin[3]));
                this.InternalComFeedGridView.Margin = new Thickness(Double.Parse(margin[0]), Double.Parse(margin[1]), Double.Parse(margin[2]), Double.Parse(margin[3]));
            }
            catch (Exception)
            {
                this.itemGridView.Margin = new Thickness(0, 0, 0, 0);
            }
        }

        /// <summary>
        /// Invoked when a group header is clicked.
        /// </summary>
        /// <param name="sender">The Button used as a group header for the selected group.</param>
        /// <param name="e">Event data that describes how the click was initiated.</param>
        void Header_Click(object sender, RoutedEventArgs e)
        {
            // Determine what group the Button instance represents
            var group = (sender as FrameworkElement).DataContext;

            // Navigate to the appropriate destination page, configuring the new page
            // by passing required information as a navigation parameter
            this.Frame.Navigate(typeof(GroupDetailPage), ((SampleDataGroup)group).UniqueId);
        }

        /// <summary>
        /// Invoked when an item within a group is clicked.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is snapped)
        /// displaying the item clicked.</param>
        /// <param name="e">Event data that describes the item clicked.</param>
        async void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            // Navigate to the appropriate destination page, configuring the new page
            // by passing required information as a navigation parameter
            String uri = ((SampleDataItem)e.ClickedItem).Description;
           
            var option = new Windows.System.LauncherOptions();
            if (((SampleDataItem)e.ClickedItem).Content != "")
            {
                option.PreferredApplicationPackageFamilyName = ((SampleDataItem)e.ClickedItem).Content;
                option.PreferredApplicationDisplayName = ((SampleDataItem)e.ClickedItem).Title;
            }
            if (uri.ToUpper().Contains("daipp://windowsStoreWin8app".ToUpper()) && !uri.ToUpper().Contains("storeid=".ToUpper()))
            {
                uri += "?storeid=" + ((SampleDataItem)e.ClickedItem).Content;
            }
            try
            {
                //If the URI is well formated go on, if not display the error message.                
                if (Uri.IsWellFormedUriString(uri, UriKind.Absolute) || uri.ToUpper().Contains("daipp://".ToUpper()))
                {
                    await Windows.System.Launcher.LaunchUriAsync(new Uri(uri), option);
                }
                else
                {
                    ((SampleDataItem)e.ClickedItem).Subtitle = String.Format(App.errorURIMessage, uri);
                    if (((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed)
                    {
                        ((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed = true;
                    }
                }
            }
            catch (FormatException)
            {
                ((SampleDataItem)e.ClickedItem).Subtitle = String.Format(App.errorURIMessage, ((SampleDataItem)e.ClickedItem).Description);
                if (((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed)
                {
                    ((SampleDataItem)e.ClickedItem).ErrorAlreadyDisplayed = true;
                }
            }
        }

        /// <summary>
        /// Invoked when an item within a RSS feed is clicked.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is snapped)
        /// displaying the item clicked.</param>
        /// <param name="e">Event data that describes the item clicked.</param>
        async void InternalComFeedGridView_ItemClick(object sender, ItemClickEventArgs e)
        {
            // Navigate to the appropriate destination page described in the item
            if(((SampleDataItem)e.ClickedItem).Description != ""){
                var uri = new Uri(((SampleDataItem)e.ClickedItem).Description);
                if(uri.IsWellFormedOriginalString())
                    await Windows.System.Launcher.LaunchUriAsync(uri);
            }
            
        }

        internal void showError(String msg)
        {
            ErrorMessage.Text = msg;
            ErrorCanvas.Visibility = 0;
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            Application.Current.Exit();
        }
        
    }
}
